<?php

/**
 * @file
 * Drush commands related to the Only One module.
 */

use \Drupal\node\Entity\NodeType;

/**
 * Implements hook_drush_command().
 */
function onlyone_drush_command() {
  $items['onlyone-list'] = array(
    'description' => dt("Shows a content types list according to the status selected as argument."),
    'aliases' => array('ol'),
    'arguments' => array(
      'status' => 'The content type status (configured, available or not-available)',
    ),
    'examples' => array(
      'drush onlyone-list configured' => dt('List the configured content types to have Only One node.'),
      'drush onlyone-list available' => dt('List the available content types to have Only One node.'),
      'drush onlyone-list not-available' => dt('List the content types not available to have Only One node.'),
    ),
  );
  $items['onlyone-summary'] = array(
    'description' => dt('Show the status summary of all the content types.'),
    'aliases' => array('os'),
    'examples' => array(
      'onlyone-summary' => dt('Show the status summary of all the content types.'),
    ),
  );
  $items['onlyone-enable'] = array(
    'description' => dt('Configure content types to have Only One node.'),
    'arguments' => array(
      'content_type' => dt('A list of content type machine names.'),
    ),
    'aliases' => array('oen'),
    'examples' => array(
      'onlyone-enable article' => dt("Configure the content type 'article' to have Only One node."),
      'onlyone-enable article page' => dt("Configure the content types 'article' and 'page' to have Only One node."),
    ),
  );
  $items['onlyone-disable'] = array(
    'description' => dt("Configure content types to not have Only One node."),
    'arguments' => array(
      'content_type' => dt('A list of content type machine names.'),
    ),
    'aliases' => array('odis'),
    'examples' => array(
      'onlyone-enable article' => dt("Configure the content type 'article' to not have Only One node."),
      'onlyone-enable article page' => dt("Configure the content types 'article' and 'page' to not have Only One node."),
    ),
  );
  return $items;
}

/**
 * Implements hook_drush_help().
 */
function onlyone_drush_help($section) {
  switch ($section) {
    case 'meta:onlyone:title':
      return dt("Only One commands");

    case 'meta:onlyone:summary':
      return dt("Interact with the Only One module's functionalities.");

    case 'drush:onlyone-list':
      return dt("This command will create a content types list according to the status selected as argument.");

    case 'drush:onlyone-summary':
      return dt('This command will create a list with the status of all the content types.');

    case 'drush:onlyone-enable':
      return dt('This command will configure content types to have Only One node.');

    case 'drush:onlyone-disable':
      return dt("This command will configure content types to not have Only One node.");
  }
}

/**
 * Callback for the onlyone-list command.
 */
function drush_onlyone_list($status = '') {
  // Check for correct argument.
  $options = [
    'configured' => dt('Configured content types to have Only One node.'),
    'available' => dt('Available content types to have Only One node.'),
    'notavailable' => dt('Content types not available to have Only One node.'),
  ];
  if (!in_array($status, array_keys($options))) {
    if (!empty($status)) {
      drush_set_error(dt('"@status" is not a valid statement status.', ['@status' => $status]));
    }
    $status = drush_choice($options, dt('What kind of list you\'d like to print?'));
  }
  if (!empty($status)) {
    $function_name = '_drush_onlyone_' . $status;
    $function_name();
  }
}

/**
 * List the configured content types that have Only One node.
 */
function _drush_onlyone_configured() {
  $onlyone_content_types = \Drupal::config('onlyone.settings')->get('onlyone_node_types');
  if (count($onlyone_content_types)) {
    // Searching the content type name.
    $content_types = [];
    foreach ($onlyone_content_types as $content_type) {
      $content_type_object = NodeType::load($content_type);
      $content_types[$content_type] = $content_type_object->label() . ' [' . $content_type . ']';
    }
    drush_print(dt('Content types configured to have Only One node:'));
    drush_print(' ' . implode("\n ", $content_types));
  }
  else {
    drush_print(dt('There is no content type configured to have Only One node.'));
  }
}

/**
 * List the available content types to have Only One node.
 */
function _drush_onlyone_available() {
  $available_content_types = _onlyone_available_content_types();

  if (count($available_content_types)) {
    drush_print(dt('Available content types to have Only One node:'));
    drush_print(' ' . implode("\n ", $available_content_types));
  }
  else {
    drush_print(dt('There is no available content type to have Only One node.'));
  }
}

/**
 * List the content types not available to have Only One node.
 */
function _drush_onlyone_notavailable() {
  $not_available_content_types = _onlyone_not_available_content_types();

  if (count($not_available_content_types)) {
    drush_print(dt('Not available content types to have Only One node:'));
    drush_print(' ' . implode("\n ", $not_available_content_types));
  }
  else {
    drush_print(dt('All the content types are available to have Only One node.'));
  }
}

/**
 * Callback for the onlyone-summary command.
 */
function drush_onlyone_summary() {
  $available_content_types = _onlyone_available_content_types();
  $not_available_content_types = _onlyone_not_available_content_types();
  $content_types_summary = array_merge($available_content_types, $not_available_content_types);
  sort($content_types_summary);

  drush_print(dt('Status summary of all the content types:'));
  drush_print(' ' . implode("\n ", $content_types_summary));
}

/**
 * Implements drush_hook_COMMAND_validate().
 */
function drush_onlyone_enable_validate() {
  $args = func_get_args();

  if (!_drush_onlyone_validate_content_type($args)) {
    return;
  }

  $availables_content_types = array_keys(_onlyone_available_content_types());
  $not_available_content_types = array_diff($args, $availables_content_types);

  if (count($not_available_content_types)) {
    $names = implode(', ', $not_available_content_types);
    $singular = 'The following content type is not available to have Only One node: @names';
    $plural = 'The following content types are not available to have Only One node: @names';
    $message = _onlyone_drush_plural(count($not_available_content_types), $singular, $plural, array('@names' => $names));
    return drush_set_error('NON_AVAILABLE', $message);
  }
}

/**
 * Callback for the onlyone-enable command.
 */
function drush_onlyone_enable() {
  $args = array_unique(func_get_args());
  // Getting an editable config because we will get and set a value.
  $config = \Drupal::service('config.factory')->getEditable('onlyone.settings');
  $onlyone_content_types = $config->get('onlyone_node_types');
  // Array to store content types that are configured and don't need to be
  // configured one more time.
  $content_types_configured = [];
  // Adding the new values to save.
  foreach ($args as $content_type) {
    if (in_array($content_type, $onlyone_content_types)) {
      $content_types_configured[] = $content_type;
    }
    else {
      $onlyone_content_types[] = $content_type;
    }
  }
  // Printing the content types that will not be configured to have Only One
  // node (they are already configured to have Only One node).
  if (count($content_types_configured)) {
    $names = implode(', ', $content_types_configured);
    $singular = 'The following content type is already configured to have Only One node: @names';
    $plural = 'The following content types are already configured to have Only One node: @names';
    $cant = count($content_types_configured);
    $message = _onlyone_drush_plural($cant, $singular, $plural, array('@names' => $names));
    drush_log($message, 'warning');
  }
  // Printing the configured content types.
  $content_types_to_enable = array_diff($args, $content_types_configured);
  if (count($content_types_to_enable)) {
    // Saving the values in the config.
    $config->set('onlyone_node_types', $onlyone_content_types);
    $config->save();

    $names = implode(', ', $content_types_to_enable);
    $singular = 'The following content type was configured to have Only One node: @names';
    $plural = 'The following content types were configured to have Only One node: @names';
    $cant = count($content_types_to_enable);
    $message = _onlyone_drush_plural($cant, $singular, $plural, array('@names' => $names));
    drush_log($message, 'success');
  }
}

/**
 * Implements drush_hook_COMMAND_validate().
 */
function drush_onlyone_disable_validate() {
  $args = func_get_args();

  if (!_drush_onlyone_validate_content_type($args)) {
    return;
  }
}

/**
 * Callback for the onlyone-disable command.
 */
function drush_onlyone_disable() {
  $args = array_unique(func_get_args());
  // Getting an editable config because we will get and set a value.
  $config = \Drupal::service('config.factory')->getEditable('onlyone.settings');
  $onlyone_content_types = $config->get('onlyone_node_types');
  // Deleting the values.
  $content_types_not_configured = [];
  foreach ($args as $content_type) {
    if (!in_array($content_type, $onlyone_content_types)) {
      $content_types_not_configured[] = $content_type;
    }
    else {
      $index = array_search($content_type, $onlyone_content_types);
      unset($onlyone_content_types[$index]);
    }
  }
  // Printing the content types that will not be configured to not have Only One
  // node (they are already configured to not have Only One node).
  if (count($content_types_not_configured)) {
    $names = implode(', ', $content_types_not_configured);
    $singular = 'The following content type is already configured to not have Only One node: @names';
    $plural = 'The following content types are already configured to not have Only One node: @names';
    $cant = count($content_types_not_configured);
    $message = _onlyone_drush_plural($cant, $singular, $plural, array('@names' => $names));
    drush_log($message, 'warning');
  }
  // Printing the not configured content types.
  $content_types_to_disable = array_diff($args, $content_types_not_configured);
  if (count($content_types_to_disable)) {
    // Saving the values in the config.
    $config->set('onlyone_node_types', $onlyone_content_types);
    $config->save();

    $names = implode(', ', $content_types_to_disable);
    $singular = "The following content type was configured to not have Only One node: @names";
    $plural = "The following content types were configured to not have Only One node: @names";
    $cant = count($content_types_to_disable);
    $message = _onlyone_drush_plural($cant, $singular, $plural, array('@names' => $names));
    drush_log($message, 'success');
  }
}

/**
 * Helper function to validate the content type names.
 *
 * @param array $content_types
 *   An array with content type machine names.
 *
 * @return bool
 *   A boolean indicating the success of the validation.
 */
function _drush_onlyone_validate_content_type(array $content_types) {
  if (!count($content_types)) {
    $message = dt('You must enter the machine name of at least one content type.');
    drush_set_error('NON_EXISTENT', $message);
    return FALSE;
  }

  $content_types_database = \Drupal::service('entity.manager')->getStorage('node_type')->loadMultiple();
  // Creating an array with all content types.
  $content_types_list = [];
  foreach ($content_types_database as $content_type) {
    $content_types_list[] = $content_type->id();
  }

  $invalid_content_types = array_diff($content_types, $content_types_list);

  if (count($invalid_content_types)) {
    $names = implode(', ', $invalid_content_types);
    $singular = 'The following content type machine name is invalid: @names';
    $plural = 'The following content type machine names are invalid: @names';
    $message = _onlyone_drush_plural(count($invalid_content_types), $singular, $plural, array('@names' => $names));
    drush_set_error('NON_EXISTENT', $message);
    return FALSE;
  }

  return TRUE;
}

/**
 * Formats a plural string containing a count of items.
 *
 * This function ensures that the string is pluralized correctly. Since dt() is
 * called by this function, make sure not to pass already-localized strings to
 * it.
 *
 * For example:
 * @code
 *   $output = _onlyone_drush_plural($node->comment_count, '1 comment', '@count comments');
 * @endcode
 *
 * @param int $count
 *   The item count to display.
 * @param string $singular
 *   The string for the singular case. Make sure it is clear this is singular,
 *   to ease translation (e.g. use "1 new comment" instead of "1 new"). Do not
 *   use @count in the singular string.
 * @param string $plural
 *   The string for the plural case. Make sure it is clear this is plural, to
 *   ease translation. Use @count in place of the item count, as in
 *   "@count new comments".
 * @param array $args
 *   An associative array of replacements to make after translation. Instances
 *   of any key in this array are replaced with the corresponding value.
 *   Based on the first character of the key, the value is escaped and/or
 *   themed. See format_string(). Note that you do not need to include @count
 *   in this array; this replacement is done automatically for the plural case.
 * @param array $options
 *   An associative array of additional options. See dt() for allowed keys.
 *
 * @return string
 *   A translated string.
 *
 * @see dt()
 */
function _onlyone_drush_plural($count, $singular, $plural, array $args = [], array $options = []) {
  $args['@count'] = $count;
  if ($count == 1) {
    return dt($singular, $args, $options);
  }

  // Get the plural index through the gettext formula.
  $index = (function_exists('locale_get_plural')) ? locale_get_plural($count, isset($options['langcode']) ? $options['langcode'] : NULL) : -1;
  // If the index cannot be computed, use the plural as a fallback (which
  // allows for most flexiblity with the replaceable @count value).
  if ($index < 0) {
    return dt($plural, $args, $options);
  }
  else {
    switch ($index) {
      case "0":
        return dt($singular, $args, $options);

      case "1":
        return dt($plural, $args, $options);

      default:
        unset($args['@count']);
        $args['@count[' . $index . ']'] = $count;
        return dt(strtr($plural, array('@count' => '@count[' . $index . ']')), $args, $options);
    }
  }
}
